\name{baseini}
\alias{baseini}
\title{Initial Values for the Baseline Function}
\description{This function can be used to derive initial values for the
baseline function which are required by two other functions: 
\code{\link{puldet}} and \code{\link{pulest}}.}
\usage{baseini(x, y, data, puloc, method=c("shift", "select"), 
     smooth=c("spline", "loess"), control=list(shift=-1, spline=list(nb=~x, 
     rk=cubic(x), spar="v", limnla=c(-1.5, 0)), loess=list(span=0.75), 
     select=list(npl=2, npr=3)))}
\arguments{
\item{x}{a vector of observation time points.}  
\item{y}{a vector of hormone concentrations.} 
\item{data}{a data frame containing the variables occurring in the \var{x} 
and \var{y} arguments. If this option is not specified,  the variables 
should be on the search list. Missing values are not allowed.} 
\item{puloc}{a vector of pulse locations.}
\item{method}{a character string.  If \option{method="shift"}, a 
smooth curve will be fit to all observations using a smoothing method 
specified by the \var{smooth} argument, and then the whole curve will be 
shifted downward; if \option{method="select"}, a smooth curve will be 
fit to the selected observations without those around pulse locations.}
\item{smooth}{a character string specifying which smoothing method 
will be used: 

\option{smooth="spline"} for smoothing spline and 
\option{smooth="loess"} for \code{\link{loess}}.}
\item{control}{a list of control parameters for different \var{method} 
and \var{smooth} options. Names of the list are \var{shift}, \var{spline}, 
\var{select}, and \var{loess}. When \option{method="shift"}, \var{shift} 
specifies the amount of shifting downward with a nonnegative value; the
default \option{shift=-1} asks the program to estimate the amount of 
shift. When 
\option{method="select"}, the \var{select} list in the control argument
decides the number of observation points to be eliminated on the
left (\var{npl}) and on the right (\var{npr}) of each pulse location.
When \option{smooth="spline"}, as in \code{\link{pcp}}, the \var{spline} 
list in the control argument specifies the null space, reproducing kernel, and 
method and range for estimating the smoothing parameter. 
When \option{smooth="loess"}, the \var{loess} list in the control argument
specifies the span for the \code{\link{loess}} function.}
}
\details{When \option{smooth="spline"}, the \code{\link{ssr}} function
in the \pkg{\link{assist}} package is used to fit the spline function.
See \url{cran.us.r-project.org/doc/packages/assist.pdf}
for more information about \var{nb}, \var{rk}, \var{spar} and 
\var{limnla}. The appropriate choices of \var{npl} and \var{npr} depend 
on the sampling rate as well as prior knowledge about the infusion and
decay rates.}
}
\value{Estimated baseline function evaluated at \var{x}.}
\references{
    Wang, Y. and Ke, C. (2002), \emph{ASSIST: A Suite of S-plus 
    functions Implementing Spline smoothing Techniques}. Available at 
    \url{cran.us.r-project.org/src/contrib/PACKAGES.html}. 
    Manual for the ASSIST package is available at 
    \url{www.pstat.ucsb.edu/faculty/yuedong/software}.

    Yang, Y. (2002), \emph{Detecting Change Points and Hormone Pulses 
    Using Partial Spline Models}, Ph.D. Thesis, University of 
    California-Santa Barbara, Dept. of Statistics and Applied Probability.
}
\author{Yu-Chieh Yang, Anna Liu, Yuedong Wang}
\seealso{\code{\link{loess}}, \code{\link{ssr}}}
\examples{
bl1 <- baseini(time, conc, data=acth, puloc=pl1, method="shift", smooth="spline")
bl2 <- baseini(time, conc, data=acth, puloc=pl1, method="select", smooth="spline")
bl3 <- baseini(time, conc, data=acth, puloc=pl1, method="shift", smooth="loess")
bl4 <- baseini(time, conc, data=acth, puloc=pl1, method="select", smooth="loess")
}
