\name{pcp}
\alias{pcp}
\title{Potential Change Points}
\description{Detect change points to the $d$th derivative of the 
mean function using partial smoothing spline models.}
\usage{pcp(x, y, data, d=1, spline = list(nb=~x, rk=cubic(x)), 
    spar="v", limnla=c(-10, 3), alpha)}
\arguments{
\item{x}{a vector of independent variable.}  
\item{y}{a vector of dependent variable.} 
\item{data}{a data frame containing the variables occurring in the \var{x} 
and \var{y} arguments. If this option is not specified,  the variables 
should be on the search list. Missing values are not allowed.} 
\item{d}{the order of change points. \option{d=1} corresponds to 
change point to the first derivative which is fixed in the 
\code{\link{pulini}} function.}
\item{spline}{a list with names \var{nb} and \var{rk}, with \var{nb}
a one-sided formula specifying the basis of the null space of the 
smoothing spline functions (default is \option{nb $=\sim$ x}) and 
\var{rk} an expression specifying the reproducing kernel of the 
orthognal complementary space to the null space (defult is 
\option{cubic(x)}).}
\item{spar}{smoothing parameter selection method. \option{spar="v"} 
for GCV (the default) and \option{spar="m"} for GML.} 
\item{limnla}{a vector of length one or two, specifying a search range 
for log10(n*lambda), where lambda is the smoothing  parameter and
n is the sample size. If it is a single value, the smoothing
parameter will be fixed at this value. Default is c(-10, 3).}
\item{alpha}{significance level for testing hypotheses that the 
coefficients of the terms representing potential change points 
in the partial smoothing spline model equal to zero.}
\details{The mean function is assumed to be smooth, except for some 
potential change points to the derivatives. The smooth function plus 
representations of those potential change points is called a partial
spline model (Wahba, 1990). We use the methods proposed in Yang (2002)
to detect potential change points. The \code{ssr} function in the
\pkg{\link{assist}} package is used to fit partial spline models.
See \url{cran.us.r-project.org/doc/packages/assist.pdf}
for more information about \var{nb}, \var{rk}, \var{spar} and 
\var{limnla}.}
\value{a vector of pulse locations.}
\references{
    Wahba, G. (1990), \emph{Spline Models for Observational Data}, 
    SIAM, CBMS-NSF Regional Conference Series in Applied Mathematics,
    Vol. 59, Philadelphia.

    Wang, Y. and Ke, C. (2002), \emph{ASSIST: A Suite of S-plus 
    functions Implementing Spline smoothing Techniques}. Available at 
    \url{cran.us.r-project.org/src/contrib/PACKAGES.html}. 
    Manual for the ASSIST package is available at 
    \url{www.pstat.ucsb.edu/faculty/yuedong/software}.

    Yang, Y. (2002), \emph{Detecting Change Points and Hormone Pulses 
    Using Partial Spline Models}, Ph.D. Thesis, University of 
    California-Santa Barbara, Dept. of Statistics and Applied Probability.
}
\author{Yu-Chieh Yang, Anna Liu, Yuedong Wang}
\seealso{\code{\link{pulini}}, \code{\link{ssr}}}
\examples{pl3 <- pcp(time, conc, data=acth, alpha=0.6)}
