\name{pulest}
\alias{pulest}
\title{Pulse Estimation}
\description{This function accomplishes the third step of the 
algorithm in Yang, Liu and Wang (2004). It estimates all parameters 
in the model.} 
\usage{pulest(x, y, data, baseline=list(nb=~x, rk=cubic(x)), 
       puloc, start, type = c("dblexp", "user"), 
       params=list(infrate="common", decrate="mixed", amplitude="mixed"), 
       pulse.usr=list(shape, parlist, devlist), 
       weights = NULL, correlation = NULL, control=pul.control(IDF=1))}

\arguments{
\item{x}{a vector of observation time points.}  
\item{y}{a vector of hormone concentrations.} 
\item{data}{a data frame containing the variables occurring in the \var{x} 
and \var{y} arguments. If this option is not specified,  the variables 
should be on the search list. Missing values are not allowed.} 
\item{baseline}{a list specifies the model space for the baseline function.
\var{nb} is a one-sided formula specifying the basis of the null 
space. \var{rk} is an expression specifying the reproducing kernel of the 
orthogonal complementary space to the null space. The default options
of \var{nb} and \var{rk} correspond to the cubic spline. 
If \option{baseline=0}, the baseline function is fixed at its 
initial input.}  
\item{start}{a list of initial values. It includes two components:
 \var{fixed}, a vector of starting values for the \emph{common} 
and \emph{fixed} parameters specified in the argument \var{params} or 
in the component \var{parlist} of the argument \var{pulse.usr} (note
that the \emph{order} is very important: the initial values for \emph{common}
parameters appear before those for \emph{fixed} parameters); \var{Inif}, 
initial baseline estimates evaluated at \var{x}.}
\item{type}{a character string specifies a predefined type of the 
pulse shape functions. If \option{type="dblexp"}, a double exponential function
will be used as the pulse shape function. If \option{type="user"}, the user can
input the shape function and parameters in the argument 
\var{pulse.usr}.}
\item{params}{when \option{type="dblexp"}, \var{params} is a list of 
three components, \var{infrate}, \var{decrate} and \var{amplitude}, 
which correspondingly specifies models to be assumed for the pulse 
infussion rate, pulse decay rate and pulse amplitude parameters. Current 
possible options are \dQuote{common} for \var{infrate}; \dQuote{common}, 
\dQuote{fixed} and \dQuote{mixed} for pulse \var{decrate}; \dQuote{common},
\dQuote{fixed}, \dQuote{mixed} and \dQuote{random} for \var{amplitude}.
Here \dQuote{common} means that a common parameter is assumed in all pulses; 
\dQuote{fixed} means that different deterministic parameters are assumed for 
all pulses; \dQuote{random} means that the parameters are assumed to be from a 
normal distribution with mean 0; \dQuote{mixed} means that the paramters 
are assumed to be from a normal distribution with a unknown constant mean.
Current program assumes that all random parameters are independent.}
\item{pulse.usr}{when \option{type="user"}, \var{pulse.usr} is a 
list of three components, \var{shape}, \var{parlist}, 
and \var{devlist}, which correspondingly specify the pulse shape, 
pulse parameters, and first derivatives of the pulse 
shape function with respect to the parameters. 
\var{shape} is a formula. \var{parlist} is a list of character 
vectors with four components: \var{common}, \var{random},
\var{fixed}, and \var{p}.
The component \var{common} specifies fixed parameters used 
in the pulse formula which are common for all pulses; the 
component \var{random} specifies random parameters in the 
pulse formula; and the component \var{fixed} specifies 
fixed parameters which are different for each pulse. 
Current program assumes that all random parameters are independent.
The fourth component \var{p} is used to specify the 
parameter name of the pulse locations. 
\var{devlist} is a list of the derivatives in the order of 
the common, random parameters and fixed parameters.
For each type of parameters, the order should be consistent 
with the order of the parameters supplied in \textt{palist}.} 
\item{weights}{an optional varFunc object or one-sided formula describing 
the heteroscedasticity structure. See the documentation on 
varClasses in the \pkg{nlme} package for a description of the available 
varFunc classes. Defaults to NULL, corresponding to homoscesdatic errors.} 
\item{correlation}{an optional corStruct object describing the 
correlation structure. See the documentation of corClasses 
in the \pkg{nlme} package for a description of the available corStruct 
classes. Defaults to NULL, corresponding to no correlations.}
\item{control}{a list of control values for the estimation algorithm to 
replace the default values returned by the function \code{\link{pulest.control}}.}
}
\details{\code{\link{pulest}} estimates all parameters using the 
double-penalized log-likelihood and the approximate profile-likelihood 
(Yang, Liu and Wang, 2004). 
}
\value{a list which contains 
\item{tbl}{a summary matrix of the fit with columns corresponding to 
the number of pulses (NP), $BIC$ (BIC), $RIC$ (RIC), $AIC$ (AIC), $GCV$ 
(GCV), total degrees of freedom (DF), degrees of freedom of the baseline 
function (df(base)), smoothing parameters in $\log_{10}$ scale (nlaht), 
and residual sum of squares (RSS).}
\item{coef}{estimated parameters.}
\item{Var}{Covariance matrix of the estimated parameters.}
\item{baseFitted}{estimates of the baseline function.} 
\item{nlmeFitted}{estimates of the hormone secretion with baseline subtracted.}
\item{nlmeObj}{nlme object of the \code{\link{nlme}} function if it is used.} 
\item{gnlsObj}{gnls object of the \code{\link{gnls}} function if it is used.}
\item{callxObj}{object returned from the inside function NLMEcallx or GNLScallx.
}
\item{nlme}{whether nlme is used in the fitting.}
}
\references{
    Pinheiro, J. and Bates, D. M. (2000), \emph{Mixed-effects Models in 
    S and S-plus}, Springer, New York.

    Yang, Y. (2002), \emph{Detecting Change Points and Hormone Pulses 
    Using Partial Spline Models}, Ph.D. Thesis, University of 
    California-Santa Barbara, Dept. of Statistics and Applied Probability.

    Yang, Y. and Liu, A. and Wang, Y., (2004), \emph{Detecting 
    Pulsatile Hormone Secretions Using Nonlinear Mixed Effects 
    Partial Spline Models}. 
    Available at \url{www.pstat.ucsb.edu/faculty/yuedong/research}.

    Wang, Y. and Ke, C. (2002), \emph{ASSIST: A Suite of S-plus 
    functions Implementing Spline smoothing Techniques}. Available at 
    \url{cran.us.r-project.org/src/contrib/PACKAGES.html}. 
    Manual for the ASSIST package is available at 
    \url{www.pstat.ucsb.edu/faculty/yuedong/software}.
}
\author{Yu-Chieh Yang, Anna Liu, Yuedong Wang}
\seealso{\code{\link{baseini}}, \code{\link{nlme}}, \code{\link{pul.control}},
\code{\link{puldec}}, \code{\link{ssr}}, \code{\link{summary.pulest}}}
\examples{
fit.cmm <- pulest(time, conc, data=acth, type=c("dblexp"),
    puloc=pl5, start=list(fixed=fix.ini, Inif=bl1),
    params=list(infrate="common", decrate="mixed", amplitude="mixed"),
    control =list(pul=list(IDF=1)))
}