\name{pulini}
\alias{pulini}
\title{Identify Initial Pulse Locations}
\description{This function accomplishes the first step of the 
algorithm in Yang, Liu and Wang (2004). 
It finds potential pulse locations. The function itself can also be 
used for pulse detection.}
\usage{pulini(x, y, data, method = c("pcp", "CLUSTER"), alpha,
     control=list(pcp=list(spline=list(nb=~x, rk=cubic(x)), spar="v", 
     limnla=c(-10, 3)), cluster=list(sd=mean(y)*0.07, nnadir=2, npeak=3)))}
\arguments{
\item{x}{a vector of observation time points.}  
\item{y}{a vector of hormone concentrations.} 
\item{data}{a data frame containing the variables occurring in the \var{x} 
and \var{y} arguments. If this option is not specified,  the variables 
should be on the search list. Missing values are not allowed.} 
\item{method}{the method to be used for identifying initial pulse
locations. If \dQuote{pcp}, the change point method based on partial 
smoothing spline models is used to detect pulse locations as change 
points to the first derivative of the mean function. If \dQuote{CLUSTER},  
the CLUSTER method proposed by Veldhuis and Johnson (1986) is used.} 
\item{alpha}{for \option{method="pcp"}, \var{alpha} controls the 
significance level of a potential change point; for 
\option{method="CLUSTER"}, 
\var{alpha} controls the significance level of the $t$ test.} 
\item{control}{A list of two components, \var{pcp} and \var{cluster}, 
to replace the default values in the \code{\link{pcp}} and 
\code{\link{CLUSTER}} functions.} 
}
\details{
\code{pulini} is a wrapper of two other functions, \code{\link{pcp}} and 
\code{\link{CLUSTER}}. See these two functions for details about control 
options. Larger \var{alpha} leads to more identified pulses, thus increases 
false positive rate and decreases false negative rate. 
\code{\link{CLUSTER}} is faster than \code{\link{pcp}}, however, its false 
negative rate is usually a bit larger.} 
\value{a vector of pulse locations.}
\references{
    Veldhuis, J. D. and Johnson, M. L. (1986), Cluster analysis: 
    a simple versatile and robust algorithm for endocrine pulse detection,
    \emph{American Journal of Physiology}, \bold{250}, E486-E493.

    Yang, Y. (2002), \emph{Detecting Change Points and Hormone Pulses 
    Using Partial Spline Models}, Ph.D. Thesis, University of 
    California-Santa Barbara, Dept. of Statistics and Applied Probability.

    Yang, Y. and Liu, A. and Wang, Y., (2004), \emph{Detecting 
    Pulsatile Hormone Secretions Using Nonlinear Mixed Effects 
    Partial Spline Models}. 
    Available at \url{www.pstat.ucsb.edu/faculty/yuedong/research}.
}
\author{Yu-Chieh Yang, Anna Liu, Yuedong Wang}
\seealso{\code{\link{CLUSTER}}, \code{\link{pcp}}}
\examples{
pl1 <- pulini(time, conc, data=acth, method="pcp", alpha=0.6)
pl2 <- pulini(time, conc, data=acth, method="CLUSTER", alpha=.2,
              control=list(cluster=list(sd=.05*mean(acth$conc))))
}

